module Main where
import Test.HUnit
import qualified SQLModel as SQL
import SQLParser
import Text.ParserCombinators.Parsec

testSimple = TestCase $ assertParse "Simple select"
  [SQL.Indenter "SELECT", SQL.Text "x"] 
  "Select x" 

testFrom = TestCase $ assertParse "With from"
  [SQL.Indenter "SELECT", SQL.Text "x", SQL.Indenter "FROM", SQL.Text "y"] 
  "Select x from y" 

testParens = TestCase $ assertParse "Parens"
  [SQL.Indenter "SELECT", SQL.Text "x", SQL.Indenter "FROM", 
  	SQL.Parens [
  		SQL.Indenter "SELECT",  SQL.Text "z",  SQL.Indenter "FROM", SQL.Text "y"
  	]
  ] 
  "Select x from (select z from y)" 

testQuestion =  TestCase $ assertParse "?" [SQL.Text "?"] "?"

testQuertionParens  =  TestCase $ assertParse "(?)" 
	[SQL.Parens [ SQL.Text "?"]]
	"(A.COMPANYID=?)"


testQuertionWhere  =  TestCase $ assertParse "Where?" 
	[SQL.Indenter "WHERE", SQL.Parens [ SQL.Text "A.COMPANYID", SQL.Text "=", SQL.Text "?"]] 
	"WHERE (A.COMPANYID=?)"


testReal = TestCase $ assertParse "Real"
	[
		SQL.Indenter "SELECT", SQL.Text "A.NAME", 
		SQL.Text ",", SQL.Text "A.LASTNAME", SQL.Text ",",
		SQL.Text "101090",
		SQL.Indenter "FROM", 
		SQL.Text "EMPLTABLE",
		SQL.Text "A",
		SQL.Text "WITH",
		SQL.Parens [SQL.Text "NOLOCK"],
		SQL.Indenter "WHERE",
		SQL.Parens [
			SQL.Text "A.DATAAREAID", SQL.Text "=", SQL.Text "'dat'"			
		],
		SQL.Text "AND",
		SQL.Text "EXISTS",
		SQL.Parens [
			SQL.Indenter "SELECT",  SQL.Text "'x'", SQL.Indenter "FROM",
			SQL.Text "UKS_VILLAGEPOSITION",  SQL.Text "B",
			SQL.Text "WITH", SQL.Parens [SQL.Text "NOLOCK"],
			SQL.Indenter "WHERE",
			SQL.Parens [
				SQL.Parens  [SQL.Text "B.DATAAREAID", SQL.Text "=", SQL.Text "'dat'"]
			]
		],
		SQL.Text "OPTION",
		SQL.Parens [SQL.Text "FAST", SQL.Text "2"]
	]
	(
		"SELECT A.NAME,A.LASTNAME, 101090 FROM EMPLTABLE A WITH( NOLOCK)" 
		++
    	" WHERE (A.DATAAREAID='dat') AND EXISTS "
    	++
    	"(SELECT 'x' FROM UKS_VILLAGEPOSITION B WITH( NOLOCK) " 
    	++
    	" WHERE ((B.DATAAREAID='dat')))  "
		++
		" OPTION(FAST 2)"
	)
main = runTestTT $ TestList 
	[
		testSimple, testFrom, testParens, testQuestion, 
	 	testQuertionParens, testQuertionWhere, testReal
	]

assertParse descr sample source = 
	case (testParse source)  of
		Right result -> assertEqual descr sample result 
		Left err -> assertFailure (descr ++ ":" ++ (show err))

testParse str =
     parse sqlParser "" str 

